"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.analyze = void 0;
function analyze(instruction) {
    const stack = [instruction];
    const features = { data: false, revivable: false };
    let item;
    while ((item = stack.pop()) !== undefined) {
        if (item.type === 'ProcessingInstruction') {
            features.data = true;
            if (item.tag) {
                features[item.tag] = true;
            }
            if (item.tag === 'attr' && item.attributes && hasDynamicAttributes(item.attributes)) {
                features[`dynamic-attr`] = true;
            }
            else if (item.tag === 'css' && item.value && hasDynamicCssValue(item.value)) {
                features[`dynamic-css`] = true;
            }
            else if (item.id) {
                features[`dynamic-${item.tag}`] = true;
            }
        }
        else if (item.id) {
            features[`dynamic-tag`] = true;
        }
        if (item.children) {
            stack.push(...item.children);
        }
    }
    features.revivable = !!Object.keys(features).find(k => k.indexOf('dynamic') >= 0);
    return features;
}
exports.analyze = analyze;
function hasDynamicAttributes(attrs) {
    return !!Object.keys(attrs).find(k => attrs[k].id);
}
function hasDynamicCssValue(value) {
    const arr = Array.isArray(value) ? value : [value];
    return !!arr.find(item => !!item.id);
}
//# sourceMappingURL=transpile.analyze.js.map